import logging
import os
import asyncio

import requests

from pscr_common.aws import get_boto3_client

_logger = logging.getLogger(__name__)

kc_legacy_env_mapping = {
    "kcdev01": "dev",
    "kctest03": "staging",
    "prd5use1": "usprod",
    "prd1apne1": "apjprod",
    "prd1apse2": "ausprod",
    "prd1ec1": "euprod",
}


class RMSSegmentController:
    def __init__(self, config, metrics, reporter):
        self.config = config
        self.ssm_client = get_boto3_client("ssm")
        self.metrics = metrics
        self.reporter = reporter

    def get_drec_aggregate_segment(self):
        search_segment_url = f'https://{self.config.get("dre_c", "host_name")}/rules/private/v3/segments/_search'
        search_segment_request = {
            "criteria": {"external_urn": [self.config.get("dre_c", "external_urn")]}
        }
        response = self.call_drec_rms_segment_api(
            search_segment_url, search_segment_request
        )
        search_segment_response = response.json()
        if len(search_segment_response["results"]) == 0:
            self.metrics.add_error_metrics("rms.no_data")
            if self.reporter is not None:
                asyncio.get_event_loop().run_until_complete(self.reporter.report_now())
            raise RuntimeError(f"{search_segment_url} returned no data")

        return search_segment_response["results"][0]["segment_id"]

    def get_drec_aggregate_segment_data(self, segment_id):
        url = f'https://{self.config.get("dre_c", "host_name")}/rules/private/v3/segments/{segment_id}/data/_search'
        segment_data_search_request = {
            "criteria": {
                "version": [self.config.get("dre_c", "version")],
                "platform": [self.config.get("dre_c", "platform")],
                "architecture": [self.config.get("dre_c", "architecture")],
                "distribution": [self.config.get("dre_c", "distribution")],
            }
        }
        response = self.call_drec_rms_segment_api(url, segment_data_search_request)
        segment_data_search_response = response.json()
        if len(segment_data_search_response["results"]) == 0:
            self.metrics.add_error_metrics("rms.no_data")
            if self.reporter is not None:
                asyncio.get_event_loop().run_until_complete(self.reporter.report_now())
            raise RuntimeError(f"{url} returned no data")

        return segment_data_search_response["results"][0]

    def call_drec_rms_segment_api(self, url, request):
        if os.getenv("RUNTIME_ENVIRONMENT_TYPE") == "k8s" and bool(
            os.getenv("DRE_C_USE_INGRESS_HOST", "True")
        ):
            _logger.info("Running on K8 Platform")
            auth_token = (
                os.getenv("API_KEY", "RMS_SERVICE_CONNECTOR_SECRET")
                + "/"
                + os.getenv("CONNECTOR_ID", "RMS_SERVICE_CONNECTOR_ID")
            )
        else:
            auth_token = self.get_auth_token()

        headers = {
            "Content-Type": "application/json",
            "X-Auth-Token": auth_token,
        }
        response = requests.post(url, json=request, headers=headers, timeout=600)
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as err:
            self.metrics.add_error_metrics("rms.api")
            if self.reporter is not None:
                asyncio.get_event_loop().run_until_complete(self.reporter.report_now())
            _logger.exception(f"Exception {err}  while Calling {url} ")
            raise
        return response

    def get_auth_token(self):

        if os.getenv("RUNTIME_ENVIRONMENT_TYPE") == "k8s":
            env_id = kc_legacy_env_mapping.get(os.getenv("ENV_ID", "UNKNOWN"))
        else:
            env_id = self.config.get("env", "id")

        try:
            return self.get_ssm_parameter(
                f'/{env_id}/{self.config.get("dre_c", "api_key")}'
            )
        except Exception:
            _logger.exception("Error Retrieving Secrets from SSM")
            self.metrics.add_error_metrics("ssm")
            if self.reporter is not None:
                asyncio.get_event_loop().run_until_complete(self.reporter.report_now())
            raise

    def get_ssm_parameter(self, param_name):
        # Get the requested parameter
        response = self.ssm_client.get_parameters(
            Names=[
                param_name,
            ],
            WithDecryption=True,
        )
        # Store the credentials in a variable
        credentials = response["Parameters"][0]["Value"]
        return credentials
